# Generated by Django 2.2.19 on 2021-03-17 17:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('inspire_eu', '0002_inspire_eu_core'),
    ]

    operations = [
        migrations.RenameField(
            model_name='cadastralparcel',
            old_name='national_cadastal_reference',
            new_name='national_cadastral_reference',
        ),
    ]
